//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "wclSerialDevices"
#pragma resource "*.dfm"
TfmMain *fmMain;
//---------------------------------------------------------------------------
__fastcall TfmMain::TfmMain(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btClearClick(TObject *Sender)
{
  lbEvents->Items->Clear();        
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::FormDestroy(TObject *Sender)
{
  wclSerialMonitor->Stop();

  FPowerMonitor->Close();
  FPowerMonitor->Free();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btEnumClick(TObject *Sender)
{
  lvDevices->Items->Clear();

  TwclSerialDevices Devices;
  int Res = wclSerialMonitor->EnumDevices(Devices);
  lbEvents->Items->Add("Enumerate serial devices: 0x" + IntToHex(Res, 8));

  if (Res == WCL_E_SUCCESS)
  {
    for (int i = 0; i < Devices.Length; i++)
    {
      TListItem* Item = lvDevices->Items->Add();
      Item->Caption = Devices[i].FriendlyName;
      Item->SubItems->Add(BoolToStr(Devices[i].IsModem, true));
      Item->SubItems->Add(Devices[i].DeviceName);
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btStartClick(TObject *Sender)
{
  int Res = wclSerialMonitor->Start();
  if (Res != WCL_E_SUCCESS)
    lbEvents->Items->Add("Start monitoring failed: 0x" + IntToHex(Res, 8));
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btStopClick(TObject *Sender)
{
  int Res = wclSerialMonitor->Stop();
  if (Res != WCL_E_SUCCESS)
    lbEvents->Items->Add("Stop monitoring failed: 0x" + IntToHex(Res, 8));
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::wclSerialMonitorInserted(TObject *Sender,
      const TwclSerialDevice &Device)
{
  lbEvents->Items->Add("Device inserted:");
  lbEvents->Items->Add("  Friendly name: " + Device.FriendlyName);
  lbEvents->Items->Add("  Modem: " + BoolToStr(Device.IsModem, true));
  lbEvents->Items->Add("  DeviceName: " + Device.DeviceName);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::wclSerialMonitorRemoved(TObject *Sender,
      const TwclSerialDevice &Device)
{
  lbEvents->Items->Add("Device removed:");
  lbEvents->Items->Add("  Friendly name: " + Device.FriendlyName);
  lbEvents->Items->Add("  Modem: " + BoolToStr(Device.IsModem, true));
  lbEvents->Items->Add("  DeviceName: " + Device.DeviceName);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::PowerStateChanged(TObject* Sender,
  const TwclPowerState State)
{
  static bool Running = false;

  switch (State)
  {
    case psResumeAutomatic:
      if (Running)
      {
        Running = false;
        wclSerialMonitor->Start();
      }
      lbEvents->Items->Add("Power changed: psResumeAutomatic");
      break;

    case psResume:
      lbEvents->Items->Add("Power changed: psResume");
      break;

    case psSuspend:
      if (wclSerialMonitor->Monitoring)
      {
        Running = true;
        wclSerialMonitor->Stop();
      }
      lbEvents->Items->Add("Power changed: psSuspend");
	  break;

	case psPowerStatusChanged:
	  lbEvents->Items->Add("Power changed: psPowerStateChanged");
	  break;

	case psUnknown:
	  lbEvents->Items->Add("Power changed: psUnknown");
	  break;
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::FormCreate(TObject *Sender)
{
  FPowerMonitor = new TwclPowerEventsMonitor();
  FPowerMonitor->OnPowerStateChanged = PowerStateChanged;
  FPowerMonitor->Open();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::wclSerialMonitorStarted(TObject *Sender)
{
  lbEvents->Items->Add("Monitoring started");
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::wclSerialMonitorStopped(TObject *Sender)
{
  lbEvents->Items->Add("Monitoring stopped");
}
//---------------------------------------------------------------------------

