//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "wclSerialDevices"
#pragma resource "*.dfm"
TfmMain *fmMain;
//---------------------------------------------------------------------------
__fastcall TfmMain::TfmMain(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btEnumClick(TObject *Sender)
{
  lvDevices->Items->Clear();

  TwclUsbDevices Devices;
  int Res = wclUsbMonitor->EnumDevices(Devices);
  if (Res != WCL_E_SUCCESS)
  {
    MessageDlg("Enumeration USB devices failed. Error: 0x" + IntToHex(Res, 8),
      mtError, TMsgDlgButtons() << mbOK, 0);
  }
  else
  {
    for (int i = 0; i < Devices.Length; i++)
    {
      TListItem* Item = lvDevices->Items->Add();

      Item->Caption = Devices[i].FriendlyName;
      Item->SubItems->Add(Devices[i].HardwareId);
      Item->SubItems->Add(IntToHex(Devices[i].VendorId, 4));
      Item->SubItems->Add(IntToHex(Devices[i].ProductId, 4));
      Item->SubItems->Add(IntToHex(Devices[i].Revision, 4));
      Item->SubItems->Add(Devices[i].Manufacturer);
      Item->SubItems->Add(Devices[i].Instance);
      Item->SubItems->Add(Sysutils::GUIDToString(Devices[i].ClassGuid));
      Item->SubItems->Add(BoolToStr(Devices[i].Enabled, True));
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btClearClick(TObject *Sender)
{
  lbEvents->Clear();        
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btStartClick(TObject *Sender)
{
  int Res = wclUsbMonitor->Start();
  if (Res != WCL_E_SUCCESS)
  {
    MessageDlg("Start monitoring failed: 0x" + IntToHex(Res, 8), mtError,
      TMsgDlgButtons() << mbOK, 0);
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btStopClick(TObject *Sender)
{
  int Res = wclUsbMonitor->Stop();
  if (Res != WCL_E_SUCCESS)
  {
    MessageDlg("Stop monitoring failed: 0x" + IntToHex(Res, 8), mtError,
      TMsgDlgButtons() << mbOK, 0);
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::wclUsbMonitorInserted(TObject *Sender,
      const TwclUsbDevice &Device)
{
  lbEvents->Items->Add("Device inserted: " + Device.Instance);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::wclUsbMonitorRemoved(TObject *Sender,
      const TwclUsbDevice &Device)
{
  lbEvents->Items->Add("Device removed: " + Device.Instance);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::FormDestroy(TObject *Sender)
{
  wclUsbMonitor->Stop();        
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btEnableClick(TObject *Sender)
{
  if (lvDevices->Selected == NULL)
    ShowMessage("Select device");
  else
  {
    int Res = wclUsbMonitor->Enable(lvDevices->Selected->SubItems->Strings[2]);
    if (Res != WCL_E_SUCCESS)
      ShowMessage("Failed to enable device: 0x" + IntToHex(Res, 8));
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btDisableClick(TObject *Sender)
{
  if (lvDevices->Selected == NULL)
    ShowMessage("Select device");
  else
  {
    int Res = wclUsbMonitor->Disable(lvDevices->Selected->SubItems->Strings[2]);
    if (Res != WCL_E_SUCCESS)
      ShowMessage("Failed to disable device: 0x" + IntToHex(Res, 8));
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::wclUsbMonitorStarted(TObject *Sender)
{
  lbEvents->Items->Add("Monitoring started");
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::wclUsbMonitorStopped(TObject *Sender)
{
  lbEvents->Items->Add("Monitoring stopped");
}
//---------------------------------------------------------------------------

